!classDefinition: #CantSuspend category: 'CodigoRepetido-Ejercicio'!
Error subclass: #CantSuspend
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!


!classDefinition: #NotFound category: 'CodigoRepetido-Ejercicio'!
Error subclass: #NotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!


!classDefinition: #CustomerBookTest category: 'CodigoRepetido-Ejercicio'!
TestCase subclass: #CustomerBookTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!

!CustomerBookTest methodsFor: 'testing' stamp: 'GR 4/14/2021 16:31:11'!
test01AddingCustomerShouldNotTakeMoreThan50Milliseconds

	|customerBook|
	
	customerBook := CustomerBook new.

	
	self assert: (self measureExecutionTimeOf: [customerBook addCustomerNamed: 'John Lennon'.] isLessThan: 50).
	
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'GR 4/14/2021 16:31:27'!
test02RemovingCustomerShouldNotTakeMoreThan100Milliseconds

	| customerBook paulMcCartney |
	
	customerBook := CustomerBook new.
	paulMcCartney := 'Paul McCartney'.
	
	customerBook addCustomerNamed: paulMcCartney.

	self assert: (self measureExecutionTimeOf: [customerBook  removeCustomerNamed: paulMcCartney.] isLessThan: 100)
	
	
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'TPL 4/14/2021 19:08:58'!
test03CanNotAddACustomerWithEmptyName 

	| customerBook |
			
	customerBook := CustomerBook new.

	[ customerBook addCustomerNamed: ''.
	self fail ]
		on: Error 
		do: [ :anError | 
			self assert: anError messageText = CustomerBook customerCanNotBeEmptyErrorMessage.
			self assert: customerBook isEmpty ]! !

!CustomerBookTest methodsFor: 'testing' stamp: 'TPL 4/15/2021 00:55:51'!
test04CanNotRemoveAnInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	[ customerBook removeCustomerNamed: 'Paul McCartney'.
	self fail ]
		on: NotFound 
		do: [:anError | self checkAmountOfCustomer: customerBook equalTo: 1 andCheckIncludeOf: johnLennon]
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'TPL 4/15/2021 00:55:45'!
test05SuspendingACustomerShouldNotRemoveItFromCustomerBook

	| customerBook paulMcCartney|
	
	customerBook := CustomerBook new.
	paulMcCartney := 'Paul McCartney'.

	customerBook addAndSuspendCustomerNamed: paulMcCartney.

	self assert: (self checkSizeOfCustomerBook: customerBook total: 1 active: 0 suspended: 1).
	self assert: (customerBook includesCustomerNamed: paulMcCartney).
	

	
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'TPL 4/15/2021 00:49:35'!
test06RemovingASuspendedCustomerShouldRemoveItFromCustomerBook

	| customerBook paulMcCartney|
	
	customerBook := CustomerBook new.
	paulMcCartney := 'Paul McCartney'.

	customerBook addAndSuspendCustomerNamed: paulMcCartney.
	customerBook removeCustomerNamed: paulMcCartney.
	
	self assert: (self checkSizeOfCustomerBook: customerBook total: 0 active: 0 suspended: 0).
	self deny: (customerBook includesCustomerNamed: paulMcCartney).


	
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'TPL 4/15/2021 00:55:34'!
test07CanNotSuspendAnInvalidCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addCustomerNamed: johnLennon.
	
	[ customerBook suspendCustomerNamed: 'George Harrison'.
	self fail ]
		on: CantSuspend 
		do: [:anError | self checkAmountOfCustomer: customerBook equalTo: 1 andCheckIncludeOf: johnLennon]
! !

!CustomerBookTest methodsFor: 'testing' stamp: 'TPL 4/15/2021 00:55:25'!
test08CanNotSuspendAnAlreadySuspendedCustomer
	
	| customerBook johnLennon |
			
	customerBook := CustomerBook new.
	johnLennon := 'John Lennon'.
	customerBook addAndSuspendCustomerNamed: johnLennon.

	[ customerBook suspendCustomerNamed: johnLennon.
	self fail ]
		on: CantSuspend 
		do: [:anError | self checkAmountOfCustomer: customerBook equalTo: 1 andCheckIncludeOf: johnLennon]

! !


!CustomerBookTest methodsFor: 'auxiliares' stamp: 'TPL 4/15/2021 00:06:01'!
checkAmount: amount equalTo: aNumber andCheckInclude: collection of: element
	self assert: amount = aNumber.
	self assert: (collection includesCustomerNamed: element).! !

!CustomerBookTest methodsFor: 'auxiliares' stamp: 'TPL 4/15/2021 00:53:19'!
checkAmountOf: customerBook equalTo: aNumber andCheckInclude: collection of: element
	self assert: customerBook = aNumber.
	self assert: (collection includesCustomerNamed: element).! !

!CustomerBookTest methodsFor: 'auxiliares' stamp: 'TPL 4/15/2021 00:53:59'!
checkAmountOfCustomer: customerBook equalTo: aNumber andCheckInclude: collection of: element
	self assert: customerBook numberOfCustomers = aNumber.
	self assert: (customerBook includesCustomerNamed: element).! !

!CustomerBookTest methodsFor: 'auxiliares' stamp: 'TPL 4/15/2021 00:54:36'!
checkAmountOfCustomer: customerBook equalTo: aNumber andCheckIncludeOf: name
	self assert: customerBook numberOfCustomers = aNumber.
	self assert: (customerBook includesCustomerNamed: name).! !

!CustomerBookTest methodsFor: 'auxiliares' stamp: 'TPL 4/15/2021 00:49:22'!
checkSizeOfCustomerBook: customerBook total: aNumber active: bNumber suspended: cNumber
^((customerBook numberOfCustomers = aNumber) and: (customerBook numberOfActiveCustomers = bNumber) and: (customerBook numberOfSuspendedCustomers  = cNumber))! !

!CustomerBookTest methodsFor: 'auxiliares' stamp: 'GR 4/12/2021 21:53:19'!
measureExecutionTimeOf: anAction isLessThan: timeInMilliseconds

	| millisecondsBeforeRunning millisecondsAfterRunning |
	

	millisecondsBeforeRunning := Time millisecondClockValue * millisecond.
	anAction.
	millisecondsAfterRunning := Time millisecondClockValue * millisecond.
	
	^(millisecondsAfterRunning-millisecondsBeforeRunning) < (timeInMilliseconds * millisecond)! !


!classDefinition: #CustomerBook category: 'CodigoRepetido-Ejercicio'!
Object subclass: #CustomerBook
	instanceVariableNames: 'suspended active'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodigoRepetido-Ejercicio'!

!CustomerBook methodsFor: 'testing' stamp: 'GR 4/14/2021 16:50:09'!
addAndSuspendCustomerNamed: aName

	self addCustomerNamed: aName.
	self suspendCustomerNamed: aName.! !

!CustomerBook methodsFor: 'testing' stamp: 'NR 4/3/2019 10:14:26'!
includesCustomerNamed: aName

	^(active includes: aName) or: [ suspended includes: aName ]! !

!CustomerBook methodsFor: 'testing' stamp: 'NR 4/3/2019 10:14:26'!
isEmpty
	
	^active isEmpty and: [ suspended isEmpty ]! !


!CustomerBook methodsFor: 'initialization' stamp: 'NR 9/17/2020 07:23:04'!
initialize

	active := OrderedCollection new.
	suspended:= OrderedCollection new.! !


!CustomerBook methodsFor: 'customer management' stamp: 'NR 4/3/2019 10:14:26'!
addCustomerNamed: aName

	aName isEmpty ifTrue: [ self signalCustomerNameCannotBeEmpty ].
	((active includes: aName) or: [suspended includes: aName]) ifTrue: [ self signalCustomerAlreadyExists ].
	
	active add: aName ! !

!CustomerBook methodsFor: 'customer management' stamp: 'NR 4/3/2019 10:14:26'!
numberOfActiveCustomers
	
	^active size! !

!CustomerBook methodsFor: 'customer management' stamp: 'NR 4/3/2019 10:14:26'!
numberOfCustomers
	
	^active size + suspended size! !

!CustomerBook methodsFor: 'customer management' stamp: 'NR 9/19/2018 17:36:09'!
numberOfSuspendedCustomers
	
	^suspended size! !

!CustomerBook methodsFor: 'customer management' stamp: 'TPL 4/15/2021 01:15:12'!
removeCustomerNamed: aName 
	^self removeCustomerFromActiveAndSuspended: aName.! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:52'!
signalCustomerAlreadyExists 

	self error: self class customerAlreadyExistsErrorMessage! !

!CustomerBook methodsFor: 'customer management' stamp: 'HernanWilkinson 7/6/2011 17:51'!
signalCustomerNameCannotBeEmpty 

	self error: self class customerCanNotBeEmptyErrorMessage ! !

!CustomerBook methodsFor: 'customer management' stamp: 'NR 4/3/2019 10:14:26'!
suspendCustomerNamed: aName 
	
	(active includes: aName) ifFalse: [^CantSuspend signal].
	
	active remove: aName.
	
	suspended add: aName
! !


!CustomerBook methodsFor: 'auxiliars' stamp: 'TPL 4/15/2021 01:15:56'!
remove: anElement fromList: list
	1 to: list size do: 
		[ :index |
			anElement = (list at: index)
				ifTrue: [
					list removeAt: index.
					^ anElement 
				] 
		].
	^ NotFound signal.
! !

!CustomerBook methodsFor: 'auxiliars' stamp: 'TPL 4/15/2021 02:27:17'!
removeCustomerFromActiveAndSuspended:  aName
	[ self remove: aName fromList: active.]
		on: NotFound 
		do: [^ self remove: aName fromList: suspended.]
	
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'CustomerBook class' category: 'CodigoRepetido-Ejercicio'!
CustomerBook class
	instanceVariableNames: ''!

!CustomerBook class methodsFor: 'error messages' stamp: 'NR 4/12/2021 16:39:13'!
customerAlreadyExistsErrorMessage

	^'customer already exists!!!!!!'! !

!CustomerBook class methodsFor: 'error messages' stamp: 'NR 4/12/2021 16:39:09'!
customerCanNotBeEmptyErrorMessage

	^'customer name cannot be empty!!!!!!'! !
