!classDefinition: #OOStackTest category: 'Stack-Exercise'!
TestCase subclass: #OOStackTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStackTest methodsFor: 'test' stamp: 'HernanWilkinson 5/7/2012 11:30'!
test01StackShouldBeEmptyWhenCreated

	| stack |
	
	stack := OOStack new.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'NR 5/13/2020 13:29:55'!
test02PushAddElementsToTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'something'.
	
	self deny: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'NR 5/13/2020 13:30:01'!
test03PopRemovesElementsFromTheStack

	| stack |
	
	stack := OOStack new.
	stack push: 'something'.
	stack pop.
	
	self assert: stack isEmpty! !

!OOStackTest methodsFor: 'test' stamp: 'NR 5/13/2020 13:30:09'!
test04PopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'something'.
	stack push: pushedObject.
	
	self assert: stack pop = pushedObject! !

!OOStackTest methodsFor: 'test' stamp: 'NR 4/22/2021 17:25:26'!
test05StackBehavesLIFO

	| stack firstPushedObject secondPushedObject |
	
	stack := OOStack new.
	firstPushedObject := 'firstObject'.
	secondPushedObject := 'secondObject'.
	
	stack push: firstPushedObject.
	stack push: secondPushedObject.
	
	self assert: stack pop = secondPushedObject.
	self assert: stack pop = firstPushedObject.
	self assert: stack isEmpty 
	! !

!OOStackTest methodsFor: 'test' stamp: 'NR 5/13/2020 13:30:20'!
test06TopReturnsLastPushedObject

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'something'.
	
	stack push: pushedObject.
	
	self assert: stack top = pushedObject.
	! !

!OOStackTest methodsFor: 'test' stamp: 'NR 5/13/2020 13:30:24'!
test07TopDoesNotRemoveObjectFromStack

	| stack pushedObject |
	
	stack := OOStack new.
	pushedObject := 'something'.
	
	stack push: pushedObject.
	
	self assert: stack size = 1.
	stack top.
	self assert: stack size = 1.
	! !

!OOStackTest methodsFor: 'test' stamp: 'HAW 4/14/2017 22:48:26'!
test08CanNotPopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack pop ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'NR 5/13/2020 13:30:31'!
test09CanNotPopWhenThereAreNoObjectsInTheStackAndTheStackHadObjects

	| stack  |
	
	stack := OOStack new.
	stack push: 'something'.
	stack pop.
	
	self
		should: [ stack pop ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !

!OOStackTest methodsFor: 'test' stamp: 'HAW 4/14/2017 22:48:44'!
test10CanNotTopWhenThereAreNoObjectsInTheStack

	| stack  |
	
	stack := OOStack new.
	self
		should: [ stack top ]
		raise: Error - MessageNotUnderstood 
		withExceptionDo: [ :anError |
			self assert: anError messageText = OOStack stackEmptyErrorDescription ]
		
! !


!classDefinition: #SentenceFinderByPrefixTest category: 'Stack-Exercise'!
TestCase subclass: #SentenceFinderByPrefixTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!SentenceFinderByPrefixTest methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 16:33:39'!
checkIfError: error happensWithPrefix: aPrefix in: aFinder with: aStack
		
		self
			should: [ aFinder findSentencesStartingWith: aPrefix in: aStack]
			raise: Error - MessageNotUnderstood 
			withExceptionDo: [ :anError |
				self assert: anError messageText = error ]! !

!SentenceFinderByPrefixTest methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 16:13:14'!
generateStackWithElements: anArray
	|stack|
	
	stack := OOStack new.

	anArray do: [:a | stack push: a]	.
	
	^stack.! !

!SentenceFinderByPrefixTest methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 15:53:17'!
isStack: aStack equalTo: anotherStack
	|areEqual|
	areEqual := (aStack size == anotherStack size).
	
	[aStack isEmpty] whileFalse: [
		areEqual := areEqual and: (aStack pop == anotherStack pop).
	].
	^areEqual.! !


!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:22:06'!
test01EmptyStackShouldReturnEmptyCollection
	|finder stack|
	finder := SentenceFinderByPrefix new.
	stack := OOStack new.
	
	self assert: ((finder findSentencesStartingWith: 'something' in: stack) isEmpty)
	
	
	
	! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:21:22'!
test02NotPresentPrefixShouldReturnEmptyCollection
	|finder stack|
	finder := SentenceFinderByPrefix new.
	stack := self generateStackWithElements: #('anotherthing').
	self assert: ((finder findSentencesStartingWith: 'something' in: stack) isEmpty)
	
	
	
	! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:25:06'!
test03PrefixPresentInOneSentenceShouldReturnCollectionWithOnlyOneSentence
	|finder prefix correctSentence stack|
	
	correctSentence := 'somethingElse'.
	
	finder := SentenceFinderByPrefix new.
	stack := OOStack new.
	stack push: correctSentence.
	
	prefix := 'something'.
	
	self assert: ((finder findSentencesStartingWith: prefix in: stack) includes: correctSentence)! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:25:22'!
test04PrefixPresentInMoreThanOneSentenceShouldReturnCollectionWithFoundSentences
	|finder prefix firstSentence secondSentence thirdSentence result stack|
	
	firstSentence := 'myFirstSentence'.
	secondSentence := 'mySecondSentence'.
	thirdSentence := 'lastSentence'.	
	finder := SentenceFinderByPrefix new.
	stack := OOStack new.
	stack push: firstSentence.
	stack push: secondSentence.
	stack push: thirdSentence.
	
	prefix := 'my'.
	
	result:= finder findSentencesStartingWith: prefix in: stack.
	
	self assert: (result includes: firstSentence).
	self assert: (result includes: secondSentence)	.
	self assert: (result size == 2)! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:13:01'!
test05ShouldPersistOrderStackAfterNotFinding
	|finder prefix tempStack elementsForStack stack|

	elementsForStack := #('sentence1' 'sentence2' 'sentence3').
	
	finder := SentenceFinderByPrefix new.
	stack := self generateStackWithElements: elementsForStack.
	
	tempStack := self generateStackWithElements: elementsForStack.
	prefix := 'notFoundPrefix'.

	finder findSentencesStartingWith: prefix in: stack.
	
	self assert: (self isStack: stack equalTo: tempStack).
	! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:14:05'!
test06ShouldPersistOrderStackAfterAFinding

	|finder prefix tempStack elementsForStack stack|

	elementsForStack := #('sentence1' 'sentence2' 'sentence3').
	
	finder := SentenceFinderByPrefix new.
	stack := self generateStackWithElements: elementsForStack.
	
	tempStack := self generateStackWithElements: elementsForStack.
	prefix := 'sentence'.

	finder findSentencesStartingWith: prefix in: stack.
	
	self assert: (self isStack: stack equalTo: tempStack).
	! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:34:04'!
test07ShouldReturnErrorWhenPrefixIsEmpty
	|finder stack prefixIsEmptyError |
	
	finder := SentenceFinderByPrefix new.
	
	stack := OOStack new.
	
	prefixIsEmptyError:= SentenceFinderByPrefix finderEmptyPrefixErrorDescription.
	
	self checkIfError: prefixIsEmptyError happensWithPrefix:  '' in: finder with: stack.
	! !

!SentenceFinderByPrefixTest methodsFor: 'tests' stamp: 'GR 4/29/2021 16:34:20'!
test08ShouldReturnErrorWhenPrefixIsHasSpace
	|finder stack prefixHasSpacesError|
	
	finder := SentenceFinderByPrefix new.
	
	stack := OOStack new.
	
	prefixHasSpacesError:= SentenceFinderByPrefix finderPrefixHasSpacesErrorDescription.
	
	self checkIfError: prefixHasSpacesError happensWithPrefix:  'a Prefix with spaces' in: finder with: stack.! !


!classDefinition: #Element category: 'Stack-Exercise'!
Object subclass: #Element
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!Element methodsFor: 'operations' stamp: 'GR 4/29/2021 16:36:39'!
isEmpty

	self subclassResponsibility ! !

!Element methodsFor: 'operations' stamp: 'GR 4/29/2021 16:36:46'!
next

	self subclassResponsibility ! !

!Element methodsFor: 'operations' stamp: 'GR 4/29/2021 16:36:49'!
value

	self subclassResponsibility ! !


!classDefinition: #DefinedElement category: 'Stack-Exercise'!
Element subclass: #DefinedElement
	instanceVariableNames: 'value next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!DefinedElement methodsFor: 'initialization' stamp: 'GR 4/26/2021 21:08:25'!
initializeWith: element after: nextElement

	value := element.
	next := nextElement.! !


!DefinedElement methodsFor: 'operations' stamp: 'GR 4/26/2021 21:19:25'!
isEmpty
	^false! !

!DefinedElement methodsFor: 'operations' stamp: 'GR 4/26/2021 21:21:25'!
next

	^next! !

!DefinedElement methodsFor: 'operations' stamp: 'GR 4/26/2021 21:21:34'!
value

	^value! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'DefinedElement class' category: 'Stack-Exercise'!
DefinedElement class
	instanceVariableNames: ''!

!DefinedElement class methodsFor: 'instance creation' stamp: 'GR 4/29/2021 16:38:39'!
initializeWith: element after: nextElement
	^self new initializeWith: element after: nextElement.! !


!classDefinition: #UndefinedElement category: 'Stack-Exercise'!
Element subclass: #UndefinedElement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!UndefinedElement methodsFor: 'operations' stamp: 'GR 4/26/2021 21:17:51'!
isEmpty

	^true! !

!UndefinedElement methodsFor: 'operations' stamp: 'GR 4/26/2021 21:21:08'!
next

	^self error: OOStack stackEmptyErrorDescription.! !

!UndefinedElement methodsFor: 'operations' stamp: 'GR 4/26/2021 21:20:58'!
value

	^self error: OOStack stackEmptyErrorDescription.! !


!classDefinition: #OOStack category: 'Stack-Exercise'!
Object subclass: #OOStack
	instanceVariableNames: 'topElement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!OOStack methodsFor: 'operations' stamp: 'GR 4/26/2021 20:54:23'!
isEmpty

	^topElement isEmpty
! !

!OOStack methodsFor: 'operations' stamp: 'GR 4/29/2021 16:06:32'!
pop
	|poppedElement|
	poppedElement := topElement.
	
	topElement := topElement next.

	
	^poppedElement value.! !

!OOStack methodsFor: 'operations' stamp: 'GR 4/29/2021 16:06:25'!
push: element
	
	topElement := DefinedElement initializeWith: element after: topElement.

! !

!OOStack methodsFor: 'operations' stamp: 'GR 4/29/2021 16:05:56'!
size
	|acumSize currentElement|
	acumSize:=0.
	currentElement:= topElement.
	[currentElement isEmpty ] whileFalse: [
		currentElement := currentElement next.
		acumSize:= acumSize + 1.
	].
	^acumSize.! !

!OOStack methodsFor: 'operations' stamp: 'GR 4/26/2021 21:03:47'!
top
	
	^topElement value! !


!OOStack methodsFor: 'initialization' stamp: 'GR 4/29/2021 16:07:15'!
initialize
	topElement := UndefinedElement new.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'OOStack class' category: 'Stack-Exercise'!
OOStack class
	instanceVariableNames: ''!

!OOStack class methodsFor: 'error descriptions' stamp: 'NR 4/22/2021 17:22:41'!
stackEmptyErrorDescription
	
	^ 'Stack is empty!!!!!!'! !


!classDefinition: #SentenceFinderByPrefix category: 'Stack-Exercise'!
Object subclass: #SentenceFinderByPrefix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Stack-Exercise'!

!SentenceFinderByPrefix methodsFor: 'operations' stamp: 'TPL 4/29/2021 00:46:33'!
findSentencesStartingWith: prefix in: aStackOfSentences

	| tempStack sentencesCollection |
	
	self checkPrefixIsValid: prefix.

	sentencesCollection := OrderedCollection new.
	tempStack:= self copyStackFrom: aStackOfSentences.

	[tempStack isEmpty] whileFalse: [
		self addTo: sentencesCollection if: prefix isPrefixOf:(tempStack pop).
	].

	^sentencesCollection
	
	! !


!SentenceFinderByPrefix methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 16:39:50'!
addTo: sentencesCollection if: prefix isPrefixOf: currentSentence
		(currentSentence beginsWith: prefix) ifTrue: [
			sentencesCollection add: currentSentence.	
		].
		^sentencesCollection! !

!SentenceFinderByPrefix methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 16:41:10'!
checkPrefixHasSpaces: aPrefix
	((aPrefix indexOf: $ )~=0) ifTrue: [^self error: self class finderPrefixHasSpacesErrorDescription ].! !

!SentenceFinderByPrefix methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 16:40:52'!
checkPrefixIsEmpty: aPrefix
	(aPrefix isEmpty) ifTrue: [^self error: self class finderEmptyPrefixErrorDescription].
! !

!SentenceFinderByPrefix methodsFor: 'auxiliary methods' stamp: 'GR 4/29/2021 16:41:35'!
checkPrefixIsValid: aPrefix
	self checkPrefixIsEmpty: aPrefix.
	self checkPrefixHasSpaces: aPrefix.! !

!SentenceFinderByPrefix methodsFor: 'auxiliary methods' stamp: 'TPL 4/29/2021 00:48:28'!
copyStackFrom: aSourceStack
	|currentElement tempStack destinyStack|

	tempStack:= OOStack new.
	destinyStack:= OOStack new.

	[aSourceStack isEmpty] whileFalse: [
		currentElement:=aSourceStack pop.
		destinyStack push: currentElement.
		tempStack push: currentElement.
	].

	[tempStack isEmpty] whileFalse: [
		aSourceStack push: (tempStack pop).
	].

	^destinyStack! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

!classDefinition: 'SentenceFinderByPrefix class' category: 'Stack-Exercise'!
SentenceFinderByPrefix class
	instanceVariableNames: ''!

!SentenceFinderByPrefix class methodsFor: 'error description' stamp: 'TPL 4/28/2021 20:40:17'!
finderEmptyPrefixErrorDescription
	
	^ 'Prefix is Empty!!!!!!'! !

!SentenceFinderByPrefix class methodsFor: 'error description' stamp: 'TPL 4/28/2021 20:44:35'!
finderPrefixHasSpacesErrorDescription
	
	^ 'Prefix has spaces!!!!!!'! !
